/*
University of Sao Paulo - Institute of Mathematics and Computer Science ( ICMC - USP )
Computer Graphics - SCC 0250
Developer (PAE): Vinicius Ruela Pereira Borges
Profa: Maria Cristina Ferreira de Oliveira

Codigo 3 - Viewports
CODE 3 - Viewports

> Para compilar este codigo, execute no terminal do Linux - To compile this code, type the following command on Linux Terminal

# gcc viewport.c  -lX11 -lglut -lGL -lGLU -lm -o viewport

Para executar o codigo, digite - To run this code, type:

# ./viewport

*/

//Bibliotecas - Libraries

// Biblioteca de entrada e saida padrao de dados
#include <stdio.h>
// Biblioteca GL: Contem as funcoes padroes do OpenGl
#include <GL/gl.h>
// Biblioteca GLU : Rotinas que utilizam os comandos OpenGL de baixo nivel para executar tarefas
#include <GL/glu.h>
// Biblioteca GLUT : Criacao de janelas e tratamento de eventos de forma independente do sistema operacional
#include <GL/glut.h>
#include <math.h>

// Estrutura desenvolvida para armazenar vertices de um poligono
struct vertice
{
    GLfloat x; //ordenada x do vertice
    GLfloat y; //ordenada y do vertice
};

typedef struct vertice Vertice;

struct poligono
{
    Vertice *vertices; //poligono eh definido pelo conjunto de vertices
    GLint nroVertices; //quantidade de vertices
};

typedef struct poligono Poligono;

// Armazena valor inteiro que identifica uma janela
int window;

// Variavel que armazena o poligono do programa
Poligono *triangle;

// Funcao que desenha o poligono de acordo com definicoes do vertice
// Neste caso, define-se um quadrado... 
Poligono* criarPoligono(int type)
{
    Poligono *novo;
    novo = (Poligono *) malloc (sizeof(Poligono));
    novo->nroVertices = 3;
    novo->vertices = (Vertice *) malloc (novo->nroVertices * sizeof(Vertice));
    novo->vertices[0].x = -50.0;
    novo->vertices[0].y = -25.0;
    novo->vertices[1].x = 50.0;
    novo->vertices[1].y = -25.0;
    novo->vertices[2].x = 0.0;
    novo->vertices[2].y = 50.0f;

    return novo;
}

// Funcao que prepara a janela onde a cena grafica sera desenhada
void InitGL(int Width, int Height) {
  
    //Ajusta a cor de fundo para preto
    glClearColor(1.0f, 1.0f, 1.0f, 0.0f);

    //Carrega a matriz de projecao
    glMatrixMode(GL_PROJECTION);
    
    //Define projecao ortogonal que mapeia o poligono nas coordenadas do mundo para coordenadas da tela
    gluOrtho2D(-100.0,100.0,-100.0, 100.0);
    
    // Aloca o poligono
    triangle = criarPoligono(1); //aloca memoria para armazenar os poligonos
}

// Funcao que desenha a cena grafica na tela
void DrawGLScene() {
  
    // Limpa a tela e o Depth Buffer
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // Ajusta a cor do poligono
    glColor3f (0.0,0.4,0.2);

    // Desenhar poligono
    GLint i;
    glBegin(GL_TRIANGLE_FAN); //desenha uma linha
      for (i=0; i < triangle->nroVertices; i++)
      {
	  glVertex2f(triangle->vertices[i].x, triangle->vertices[i].y);
      }
    glEnd();
    
    // Processa as rotinas do OpenGL o mais rápido possível
    glFlush();
}

// Funcao que trata eventos do teclado
void keyboard (unsigned char key, int x, int y){
  
    int i;
  
    
    if(key == '1'){
         // Se pressionar tecla 1, viewport da esquerda eh habilitada
	 glViewport(0, 0, 300, 300);
	 
    }else{
        // Se pressionar tecla 2, viewport da direita eh habilitada
	if(key == '2'){
	    glViewport(300, 0, 300, 300);
	}else{
	    // viewport eh ajustada para o tamanho da janela GLUT
	    glViewport(0, 0, 600, 300);
	}
    }
    glutDisplayFunc(DrawGLScene);
    glutPostRedisplay();
}

// Programa principal
main(int argc, char *argv[])
{

    //Inicializa o Glut
    glutInit(&argc, argv);

    // Ajusta o display para o usuario
    glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);

    //Coordenada inicial da janela
    glutInitWindowPosition(0, 0);

    //Ajusta o tamanho da janela com 640x480
    glutInitWindowSize(600,300);

    //Nome da Janela
    window = glutCreateWindow("2D Viewing");

    // Registra a funcao de callback do teclado
    glutKeyboardFunc(&keyboard);
    
    //Funcao que prepara a cena grafica na janela com dimensoes 640 x 480
    InitGL(600, 300);

    //Desenha a cena grafica na tela
    glutDisplayFunc(DrawGLScene);
    
    //Loop de atualizacao da tela
    glutMainLoop();

    return 0;
}
